﻿#region Copyright (C) 2005-2010 Team MediaPortal

// Copyright (C) 2005-2010 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

using System.IO;
using System.Reflection;
using System.Diagnostics;
using MediaPortal.Configuration;
using MediaPortal.GUI.Library;
using MediaPortal.Util;

namespace WOLPowerManager
{
    internal static class Settings
    {
        #region Properties


        //öffentliche Variablen
        private const string PLUGIN_NAME = "WOLPowerManager";

        public static bool ManualWOL { get; set; }

        public static Action.ActionType RemoteButton { get; set; }
        public static int WaitTime { get; set; }

        public static bool UseIpInsteadDNSname { get; set; }
        public static int Port { get; set; }

        public static bool IntervalCheck { get; set; }
        public static int IntervalTime { get; set; }

        public static int ServersCount { get; set; }
        public static int PluginsCount { get; set; }
        public static string[,] Servers { get; set; }
        public static string[,] Plugins { get; set; }


        static Settings()
        {
            ServersCount=0;
            PluginsCount = 0;
        }

        #endregion

        

        public static void Load()
        {
           MediaPortal.GUI.Library.Log.Info("{0}: Settings.Load()", PLUGIN_NAME);

            using (
              MediaPortal.Profile.Settings xmlReader =
                new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "WOLPowerManager.xml")))
            {
                // WOL Settings
                ServersCount = xmlReader.GetValueAsInt(PLUGIN_NAME, "ServersCount",0);
                Servers = new string[ServersCount, 5]; //Servers[x,0] = 0:CompName|1:IP|2:MAC|3:StartupWOL|4:ServerState
                                                        // Serverstate wird nicht gespeichert, da dynamisch

                for (int i = 0; i < ServersCount; i++)
                {
                    Servers[i,0] = xmlReader.GetValueAsString(PLUGIN_NAME, "Computername"+i, "");
                    Servers[i,1] = xmlReader.GetValueAsString(PLUGIN_NAME, "IP"+i, "192.168.0.0");
                    Servers[i,2] = xmlReader.GetValueAsString(PLUGIN_NAME, "MAC"+i, "");
                    Servers[i,3] = xmlReader.GetValueAsString(PLUGIN_NAME, "StartupWOL" + i, "false");
                }

                PluginsCount = xmlReader.GetValueAsInt(PLUGIN_NAME, "PluginsCount", 0);
                Plugins = new string[PluginsCount, 4]; //Plugins[x,0] = 0:CheckBox|1:PluginName|2:CompName|3:PluginID

                for (int j = 0; j < PluginsCount; j++)
                {
                    Plugins[j, 0] = xmlReader.GetValueAsString(PLUGIN_NAME, "Check" + j, "false");
                    Plugins[j, 1] = xmlReader.GetValueAsString(PLUGIN_NAME, "Pluginname" + j, "");
                    Plugins[j, 2] = xmlReader.GetValueAsString(PLUGIN_NAME, "Servername" + j, "");
                    Plugins[j, 3] = xmlReader.GetValueAsString(PLUGIN_NAME, "PluginID" + j, "");
                }

                //Advanced
                ManualWOL = xmlReader.GetValueAsBool(PLUGIN_NAME, "ManualWOL", true);

                WaitTime = xmlReader.GetValueAsInt(PLUGIN_NAME, "WaitTime", 20);
                Port = xmlReader.GetValueAsInt(PLUGIN_NAME, "Port", 0);
                UseIpInsteadDNSname = xmlReader.GetValueAsBool(PLUGIN_NAME, "UseIP", false);
                RemoteButton = (Action.ActionType)xmlReader.GetValueAsInt(PLUGIN_NAME, "RemoteButton", (int)(Action.ActionType.ACTION_REMOTE_BLUE_BUTTON));

                IntervalCheck = xmlReader.GetValueAsBool(PLUGIN_NAME, "IntervalCheck", true);
                IntervalTime = xmlReader.GetValueAsInt(PLUGIN_NAME, "IntervalTime", 30);
             }

            WriteToLog();
        }

        public static void Save()
        {
            Log.Info("{0}: Settings.Save()", PLUGIN_NAME);

            using (
              MediaPortal.Profile.Settings xmlWriter =
                new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "WOLPowerManager.xml")))
            {
                xmlWriter.SetValue(PLUGIN_NAME, "ServersCount", Servers.GetLength(0));
                for (int i = 0; i < Servers.GetLength(0); i++)
                {
                    xmlWriter.SetValue(PLUGIN_NAME, "Computername"+i, Servers[i, 0]);
                    xmlWriter.SetValue(PLUGIN_NAME, "IP"+i, Servers[i, 1]);
                    xmlWriter.SetValue(PLUGIN_NAME, "MAC"+i, Servers[i, 2]);
                    xmlWriter.SetValue(PLUGIN_NAME, "StartupWOL" + i, Servers[i, 3]);
                }

                xmlWriter.SetValue(PLUGIN_NAME, "PluginsCount", Plugins.GetLength(0));
                for (int j = 0; j < Plugins.GetLength(0); j++)
                {
                    xmlWriter.SetValue(PLUGIN_NAME, "Check" + j, Plugins[j, 0]);
                    xmlWriter.SetValue(PLUGIN_NAME, "Pluginname"+j, Plugins[j, 1]);
                    xmlWriter.SetValue(PLUGIN_NAME, "Servername"+j, Plugins[j, 2]);
                    xmlWriter.SetValue(PLUGIN_NAME, "PluginID" + j, Plugins[j, 3]);
                }

                xmlWriter.SetValueAsBool(PLUGIN_NAME, "ManualWOL", ManualWOL);

                xmlWriter.SetValue(PLUGIN_NAME, "WaitTime", (int)WaitTime);
                xmlWriter.SetValue(PLUGIN_NAME, "Port", (int)Port);
                xmlWriter.SetValueAsBool(PLUGIN_NAME, "UseIP", UseIpInsteadDNSname);
                xmlWriter.SetValue(PLUGIN_NAME, "RemoteButton", (int)RemoteButton);

                xmlWriter.SetValueAsBool(PLUGIN_NAME, "IntervalCheck", IntervalCheck);
                xmlWriter.SetValue(PLUGIN_NAME, "IntervalTime", (int)IntervalTime);
                
            }
        }

        public static void WriteToLog()
        {
            // get all properties
            PropertyInfo[] propertyInfos = typeof(Settings).GetProperties();
            // write property names and values
            foreach (PropertyInfo propertyInfo in propertyInfos)
            {
                Log.Info("{0}: {1}  =  {2}", PLUGIN_NAME, propertyInfo.Name, propertyInfo.GetValue(null, null).ToString());
            }
        }
    }
}