#region Copyright (C) 2005-2010 Team MediaPortal

// Copyright (C) 2005-2010 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion

namespace WOLPowerManager
{
  using System;
  using System.Net;
  using System.Net.NetworkInformation;
  using System.Net.Sockets;
  using System.Text;
  using System.Threading;

  using MediaPortal.Configuration;
  using MediaPortal.Dialogs;
  using MediaPortal.GUI.Library;

  [PluginIcons("WOLPowerManager.img.WOLPowerManager_logo_enabled.png", "WOLPowerManager.img.WOLPowerManager_logo_disabled.png")]
  public class WOLPowerManager : ISetupForm, IPlugin
  {
    //MP
    public GUIDialogNotify _dialogWakeOnLanNotify;
    public GUIDialogMenu _dialogServerMenu;

    private volatile bool _doInterval = true;


    // Constructor
    public WOLPowerManager()
    {

    }

    private void IntervalStart()
    {
      Thread Interval = new Thread(new ThreadStart(this.IntervalPing));
      Interval.Start();
    }


    ///<summary> Prft welchen Powerstatus die Server haben.</summary>
    private void IntervalPing()
    {
      if (Settings.IntervalCheck)
      {
        while (this._doInterval)
        {
          for (int i = 0; i < Settings.Servers.GetLength(0); i++)
          {
            if (Settings.UseIpInsteadDNSname)
              Settings.Servers[i, 4] = this.PingHost(Settings.Servers[i, 1]).ToString();
            else
              Settings.Servers[i, 4] = this.PingHost(Settings.Servers[i, 0]).ToString();
          }
          this.UpdateGUIProperties();
          Thread.Sleep(Settings.IntervalTime * 1000);
        }
      }
    }

    /// <summary> Erneuert die GUI Labels </summary>
    private void UpdateGUIProperties()
    {
      for (int i = 0; i < Settings.Servers.GetLength(0); i++)
      {
        string serverstate = "";
        switch (Convert.ToInt64(Settings.Servers[i, 4]))
        {
          case -1:    //offline
            serverstate = "offline";
            break;
          case -2:    //Ping-Exception
            serverstate = "error";
            break;
          default:    //online Ping reply > 0ms
            serverstate = "online";
            break;
        }
        GUIPropertyManager.SetProperty("#WOLPowerManager.Servername" + i.ToString(), Settings.Servers[i, 0]);
        GUIPropertyManager.SetProperty("#WOLPowerManager.Serverstate" + i.ToString(), serverstate);
        Log.Debug("Setting GUI Property: '" + @"#WOLPowerManager.Servername" + i.ToString() + "' to '" + Settings.Servers[i, 0] + "'");
        Log.Debug("Setting GUI Property: '" + @"#WOLPowerManager.Serverstate" + i.ToString() + "' to '" + serverstate + "'");
      }
    }

    /// <summary>
    /// Sendet ein WOL Package
    /// </summary>
    /// <param name="index">Index des Profils</param>
    private static void WakeOnLan(int index)
    {
      //Umwandeln der MAC Adresse von String zu byte
      string[] arr_mac = Settings.Servers[index, 2].Split(':');
      byte[] macadresse = new byte[6];

      for (int x = 0; x < macadresse.Length; x++)
      {
        macadresse[x] = byte.Parse(arr_mac[x],
                                 System.Globalization.NumberStyles.HexNumber);
      }
      byte[] mac = macadresse;    // MAC Adresse als Byte


      /*
        Das WOL Signal wird als Broadcast verschickt.
        Es enthlt 6x den Wert FF, direkt danach folgt 16x die MAC Adresse.
      */
      UdpClient WOLclient = new UdpClient();
      WOLclient.Connect(IPAddress.Broadcast, Settings.Port);

      //Startsignal 6x(FF) + 16x(Mac-Adresse) = 102bytes
      byte[] Startsignal = new byte[] { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
      byte[] WOLSignal = new byte[102];

      // Startsignal einf&uuml;gen
      Startsignal.CopyTo(WOLSignal, 0);

      // Die Mac-Adresse wird 16x in das WOL Signal kopiert
      for (int i = 1; i <= 16; i++)
        mac.CopyTo(WOLSignal, i * 6);

      //Signal senden
      WOLclient.Send(WOLSignal, WOLSignal.Length);
    }

    /// <summary>
    /// Prft ob das Plugin in der Liste eingetragen ist und aktiviert wurde. Weckt ggf. den Server
    /// </summary>
    /// <param name="windowID">ID des Pluginfensters</param>
    public void GUIWindowManager_OnActivateWindow(int windowID)
    {
      //geht alle eingetragenen Server nacheinander durch
      for (int i = 0; i < Settings.Servers.GetLength(0); i++)
      {
        //geht alle eingetragenen Plugins  nacheinander durch
        for (int j = 0; j < Settings.Plugins.GetLength(0); j++)
        {
          //prft ob ein Server mit einem Plugin konfiguriert/gepaart wurde
          if (Settings.Plugins[j, 2] == Settings.Servers[i, 0])
          {
            //prft ob dieses Plugin nun aktiv ist
            if ((windowID.ToString() == Settings.Plugins[j, 3]) && (Settings.Plugins[j, 0] == "True"))
            {
              //weckt ggf. den Server
              this.WakeSelectedServer(i);
            }
          }
        }
      }

    }

    private void GUIWindowManager_OnNewAction(MediaPortal.GUI.Library.Action action)
    {
      try
      {
        if (action.wID == Settings.RemoteButton && Settings.ManualWOL == true)
        {
          this._dialogServerMenu = (GUIDialogMenu)GUIWindowManager.GetWindow((int)GUIWindow.Window.WINDOW_DIALOG_MENU);
          this._dialogServerMenu.SetHeading("Wake up ...");
          for (int i = 0; i < Settings.Servers.GetLength(0); i++)
          {
            if (Convert.ToInt32(Settings.Servers[i, 4]) == -1)
              this._dialogServerMenu.Add("|OFF| " + Settings.Servers[i, 0]);
          }

          for (int i = 0; i < Settings.Servers.GetLength(0); i++)
          {
            if (Convert.ToInt32(Settings.Servers[i, 4]) >= 0)
              this._dialogServerMenu.Add("| ON| " + Settings.Servers[i, 0]);
          }
          this._dialogServerMenu.DoModal(GUIWindowManager.ActiveWindow);
          //Ausgewhlter Server wird abgefragt und geweckt
          string Selected = this._dialogServerMenu.SelectedLabelText;
          for (int i = 0; i < Settings.Servers.GetLength(0); i++)
          {
            if (Selected.Remove(0, 6) == Settings.Servers[i, 0])
              this.WakeSelectedServer(i);
          }

        }
      }
      catch (Exception ex)
      {
        Log.Error(ex);
      }
    }


    /// <summary>
    /// Prfe ob Server neue IP (DHCP) erhalten hat, wenn ja Eintrag erneuern.
    /// </summary>
    /// <param name="host">Computername des zu pingenden Servers</param>
    private void CheckNewIP(string host)
    {
      IPAddress[] addresslist = null;
      //wrap the attempt in a try..catch to capture any exceptions that may occur
      try
      {
        //get the host IP from the name provided
        //address = Dns.GetHostEntry(host).AddressList[0];
        addresslist = Dns.GetHostAddresses(host);
      }
      catch (SocketException ex)
      {
        //some DNS error happened, return the message
        Log.Debug(string.Format("DNS Error: {0}", ex.Message));
      }

      if (addresslist == null) 
        return;
      foreach (IPAddress theaddress in addresslist)
      {
        for (int i = 0; i < Settings.Servers.GetLength(0); i++)
        {
          //Sucht im Array nach passendem Host-Eintrag und ndert IP falls neu
          if ((Settings.Servers[i, 0] == host) && (Settings.Servers[i, 1] != theaddress.ToString()))
          {
            Settings.Servers[i, 1] = theaddress.ToString();
            Log.Info("New IP {0} found for {1}", theaddress.ToString(), host);
          }
        }
      }
    }

    private long PingHost(string host)
    {
      try
      {
        Ping pingSender = new Ping();
        PingOptions options = new PingOptions();
        options.DontFragment = true;
        // Create a buffer of 32 bytes of data to be transmitted.
        string data = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
        byte[] buffer = Encoding.ASCII.GetBytes(data);
        int timeout = 120;
        PingReply reply = pingSender.Send(host, timeout, buffer, options);
        if (reply.Status == IPStatus.Success)
        {
          // CheckNewIP(host);
          return reply.RoundtripTime;
        }
        else
        {
          this.CheckNewIP(host); // if there is no success, check, if there is a new IP address available for the host name
          return -1;  //Ping Failed
        }
          
      }
      catch (Exception ex)
      {
        Log.Info("Ping can't be executed! It's blocked by a firewall or a network connection isn't available. Error: " + ex.Message);
        // Log.Error("Exception Error: " + ex.InnerException);
        return -2;
      }
    }


    private void WaitDialog(int timeOut, string notifyMessage)
    {
      try
      {
        //timeOut /= 10000; 

        //show
        this._dialogWakeOnLanNotify =
          (GUIDialogNotify)GUIWindowManager.GetWindow((int)GUIWindow.Window.WINDOW_DIALOG_NOTIFY);
        this._dialogWakeOnLanNotify.TimeOut = timeOut; //[in seconds]
        this._dialogWakeOnLanNotify.SetImage(GUIGraphicsContext.Skin + @"\Media\WOLPowerManager_logo_dialog.png");
        this._dialogWakeOnLanNotify.SetHeading("WOL-PowerManager");
        this._dialogWakeOnLanNotify.SetText(notifyMessage);
        this._dialogWakeOnLanNotify.DoModal(GUIWindowManager.ActiveWindow);
      }
      catch (Exception ex)
      {
        Log.Error(ex);
      }
    }

    /// <summary>
    /// Weckt den Server, falls offline. Zeigt eine Mediaportal-GUI-Meldung (Countdown bis online)
    /// </summary>
    /// <param name="index">0-basierender Index des Settings.Server-Arrays</param>
    private void WakeSelectedServer(int index)
    {
      switch (this.PingHost(Settings.Servers[index, 0]))
      {
        case -1:
          this.WaitDialog(1, string.Format("Send WOL-Package to {0} ({1})", Settings.Servers[index, 0], Settings.Servers[index, 1]));

          WakeOnLan(index);

          this.WaitDialog(Settings.WaitTime, string.Format("Waiting for answer of {0}", Settings.Servers[index, 0]));
          int ActualTime = DateTime.Now.Second;
          while (DateTime.Now.Second - ActualTime < Settings.WaitTime) //Solange pingen bis Timeout berschritten Berechnung durch: Startzeit - Neuberechnete_Aktuelle_Zeit
          {
            if (this.PingHost(Settings.Servers[index, 0]) >= 0)
            {
              //Schliee Dialog und verlasse While-Schleife 
              this._dialogWakeOnLanNotify.PageDestroy();
              return;
            }
            else
            {
              this.CheckNewIP(Settings.Servers[index, 0]); // check for new IP, if server not responding
            }
          }
          this.WaitDialog(2, string.Format("Error: {0} could not be waked up! (Timeout)", Settings.Servers[index, 0]));
          break;

        case -2:
          this.WaitDialog(2, string.Format("Error: State of {0} could not be checked! (Ping-Exception)", Settings.Servers[index, 0]));
          break;
      }
    }

    #region ISetupForm Members

    // Returns the name of the plugin which is shown in the plugin menu
    public string PluginName()
    {
      return "WOL-PowerManager";
    }

    // Returns the description of the plugin is shown in the plugin menu
    public string Description()
    {
      return "Sends WOL-Packages when entering the specified plugins";
    }

    // Returns the author of the plugin which is shown in the plugin menu
    public string Author()
    {
      return "jojo1411";
    }

    // show the setup dialog
    public void ShowPlugin()
    {
      new WOLPowerManagerConf().ShowDialog();
    }

    // Indicates whether plugin can be enabled/disabled
    public bool CanEnable()
    {
      return true;
    }

    // Get Windows-ID
    public int GetWindowId()
    {
      // WindowID of windowplugin belonging to this setup
      // enter your own unique code
      return -1;
    }

    // Indicates if plugin is enabled by default;
    public bool DefaultEnabled()
    {
      return true;
    }

    // indicates if a plugin has it's own setup screen
    public bool HasSetup()
    {
      return true;
    }

    /// <summary>
    /// If the plugin should have it's own button on the main menu of MediaPortal then it
    /// should return true to this method, otherwise if it should not be on home
    /// it should return false
    /// </summary>
    /// <param name="strButtonText">text the button should have</param>
    /// <param name="strButtonImage">image for the button, or empty for default</param>
    /// <param name="strButtonImageFocus">image for the button, or empty for default</param>
    /// <param name="strPictureImage">subpicture for the button or empty for none</param>
    /// <returns>true : plugin needs it's own button on home
    /// false : plugin does not need it's own button on home</returns>

    public bool GetHome(out string strButtonText, out string strButtonImage,
      out string strButtonImageFocus, out string strPictureImage)
    {
      strButtonText = this.PluginName();
      strButtonImage = String.Empty;
      strButtonImageFocus = String.Empty;
      strPictureImage = String.Empty;
      return false;
    }

    #endregion

    #region IPlugin Member

    /// <summary>
    /// This method will be called by mediaportal to start your process plugin
    /// </summary>
    public void Start()
    {
      Settings.Load();

      //WOL alle Server die fr WOLStartup angeklickt sind
      for (int i = 0; i < Settings.Servers.GetLength(0); i++)
      {
        if (Convert.ToBoolean(Settings.Servers[i, 3].ToString()))
          WakeOnLan(i);
      }

      //Neuer Thread PingIntervall wird hinzugefgt
      this.IntervalStart();
      GUIPropertyManager.SetProperty("#WOLPowerManager.IntervalState", Settings.IntervalCheck.ToString());

      // init server state labels
      for (int i = 0; i < Settings.Servers.GetLength(0); i++)
      {
        GUIPropertyManager.SetProperty("#WOLPowerManager.Servername" + i.ToString(), "");
        GUIPropertyManager.SetProperty("#WOLPowerManager.Serverstate" + i.ToString(), "");
      }


      //Abonent der Event-Funktion
      GUIWindowManager.OnActivateWindow += new GUIWindowManager.WindowActivationHandler(this.GUIWindowManager_OnActivateWindow);
      GUIWindowManager.OnNewAction += new OnActionHandler(this.GUIWindowManager_OnNewAction);

    }

    /// <summary>
    /// This method will be called by mediaportal to stop your process plugin
    /// </summary>
    public void Stop()
    {
      //Event-Funktion kndigen
      GUIWindowManager.OnActivateWindow -= new GUIWindowManager.WindowActivationHandler(this.GUIWindowManager_OnActivateWindow);
      GUIWindowManager.OnNewAction -= new OnActionHandler(this.GUIWindowManager_OnNewAction);

      this._doInterval = false;

      Settings.Save();
    }

    #endregion

  }
}
